###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-10 by:                                              #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
import os
import string
from cistematic.programs import supportedPrograms
from cistematic.experiments import experimentTypes, loadExperiment
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

class GenExp:
    dataPath = ''
    dbFilePath = ''
    expFilePath = ''
    expName = ''
    expType = ''
    dataset = []
    expProgs = []
    expSettings = {}
    analysisName = ''
    
    def __init__(self, path, dbPath, newexp, type='', dset='', progs='', analysis='', createExp = False):
        self.dataPath = path
        self.dbFilePath = dbPath
        self.expName = newexp
        #self.expFilePath = path + newexp + '.py'
        self.expFilePath = string.replace(dbPath,'.db','-db.py')
        self.expType = type
        self.dataset = dset
        self.expProgs = progs
        self.analysisName = newexp + analysis
        
        if createExp:
            self.createExpDB()
        
        if len(self.expProgs) == 0:
            exp = loadExperiment( self.expName , self.dbFilePath)
            self.expType = exp.experimentType
            self.expProgs = exp.getSetting('expProgs')
        
        if len(dset) == 0:
            exp = loadExperiment( self.expName , self.dbFilePath)
            self.expType = exp.experimentType
            self.dataset = exp.getDataset()
        
        self.createExpRunFile()
    
    def run(self):
        exp = loadExperiment( self.expName , self.dbFilePath)
        exp.setExternalStatus("QUEUED")
        del exp
        #runFile = open(self.expFilePath + '.run','w')
        #runFile.write('/usr/bin/python ' + self.expFilePath)
        #runFile.close()
        #
        #os.system('cd ' + self.dataPath + ' ; echo "sh ' + self.expFilePath + '.run " | at now + 1 minute')
        os.system('cd ' + self.dataPath + ' ; nohup /usr/bin/python ' + self.expFilePath + ' & ')
    
    def createExpDB():
        strexec = 'from cistematic.experiments.' + modName + ' import ' + self.expType + '\n'
        exec strexec
        strexec = 'exp = ' + self.expType + '("' + self.expName + '", "' + self.dbFilePath + '")\n'
        exec strexec
        
    def createExpRunFile(self):
        outFile = open(self.expFilePath, 'w')
        outFile.write(self.expHead())
        outFile.write(self.expInit())
        outFile.write(self.expPrograms())
        outFile.write(self.expRun())
        outFile.write(self.expAnalysis())
        outFile.write(self.expStatus())
        
        outFile.close()
    
    def expHead(self):
        strHead = '# Experiment: ' + self.expName + ' autogenerated by Cistematic GenExp 0.9.9b\n'
        
        strHead += 'from sys import path\n'
        strHead += "cisPath = '" + cisRoot + "'\n"
        strHead += "if cisPath not in path:\n\tpath.append(cisPath)\n\n"
        
        for oneProg in supportedPrograms:
            strHead += 'from cistematic.programs.' + oneProg[0] + ' import ' + oneProg[1] + '\n'
        
        return strHead + '\n'
    
    def expInit(self):
        preInitialize = ''
        useMussa = False
        useSeqcomp = False
        useSimple = True
        startingGenome = ''
        targetGenomes = []
        consGene = []
        geneIDList = []
        
        if self.expType  in ['orthology', 'phyloFoot', 'phyloTest']:
            useSimple = False
            exp = loadExperiment( self.expName , self.dbFilePath)
            consSettings = exp.getSetting('expConsConfig')
            
            (window, threshold, numseq, homologs) = consSettings[0].split(':')
            # reverse order of dataset to ultimately pick up the "last gene"
            self.dataset.reverse()
            # Let's hope we already have target genomes
            if homologs != '1' and exp.settingsHasKey('gTargets'):
                gTargets = exp.getSetting('gTargets')
                if len(gTargets) > 0:
                    targetGenomes = gTargets[0].split(':')
            del exp
            
            for geneID in self.dataset:
                (genome, gid) = geneID
                if (genome not in targetGenomes) and homologs == '1':
                    targetGenomes.append(genome)
                if startingGenome == '':
                    startingGenome = genome
                gid = gid[0]
                # we are assuming that all of these genes will be from the same 
                # genome - this is only true (or necessary) if these are not 
                # explicitely maked as "homologs" by the user
                if gid not in consGene:
                    consGene.append(gid)
                geneIDList.append((genome, gid))
            
            if homologs == '1':
                preInitialize += 'exp.createConservation()\n'
                preInitialize += 'exp.loadConservation()\n'
                preInitialize += 'paralogs = ' + str(geneIDList) + '\n'
                preInitialize += 'exp.insertHomologs(paralogs, "genExp")\n\n'
                # use the first gene as the only entry in the consGene list
                del consGene[1:]
            
            if int(numseq) > 0:
                useSeqcomp = True
            else:
                useMussa = True
        else:
            initializeArguments = str(self.dataset)
        
        strInit = ''
        
        #Need to tab everything to account for the try:/except: construct
        # strInit = 'try:\n'
        strInit += '# Initialize Experiment\n'
        
        for (expMod, expClass) in experimentTypes:
            if expClass == self.expType:
                modName = expMod
        
        strInit += 'from cistematic.experiments import loadExperiment\n\n'
        strInit += 'exp = loadExperiment("' + self.expName + '", "' + self.dbFilePath + '")\n'
        strInit += 'exp.setExternalStatus("INITIALIZING")\n'
        
        strInit += preInitialize
        
        if useMussa or useSeqcomp:
            if len(targetGenomes) > 0:
                strInit += 'exp.initialize("' + startingGenome + '", ' + str(consGene) + ', ' + str(targetGenomes) + ')\n\n'
            else:
                strInit += 'exp.initialize("' + startingGenome + '", ' + str(consGene) + ')\n\n'
            strInit += 'exp.setExternalStatus("CONSERVATION")\n'
            strInit += 'exp.computeAlignments()\n'
        else:
            strInit += 'exp.initialize(' + initializeArguments + ')\n\n'
        
        if useMussa:
            strInit += 'exp.mapMussaConservation(window=%s, threshold=%s)\n\n' % (window, threshold)
        
        if useSeqcomp:
            strInit += 'exp.mapSeqcompConservation(window=%s, threshold=%s, minSequences=%s)\n\n' % (window, threshold, numseq)
        
        return strInit
    
    def expPrograms(self):
        strProg = '# Load individual programs and settings\n'
        index = 1
        genomes = []
        
        for (genome, geneID) in self.dataset:
            if genome not in genomes:
                genomes.append(genome)
        
        for entry in self.expProgs:
            progArray = entry[:-1].split(';')
            (amod, aProg) = progArray[0].split(':')
            strProg += 'prog' + str(index) + ' = ' + aProg + '()\n'
            if len(genomes) == 1 and self.expType == 'Simple':
                strProg += 'prog' + str(index) + '.setGenome("' + genomes[0] + '")\n'
            if len(progArray) > 1:
                strProg += 'prog' + str(index) + '.setGenExpOptions(' + str(progArray[1:]) + ')\n'
            strProg += 'exp.appendProgram(prog' + str(index) + ')\n'
            index += 1
        
        return strProg + '\n'
    
    def expRun(self):
        strRun = '# Run experiment\n'
        
        strRun += 'exp.setExternalStatus("MOTIF FINDING")\n'
        strRun += 'exp.run()\n'
        
        return strRun + '\n'
    
    def expAnalysis(self):
        strAnalysis = '# Analysis Section\n'
        #strAnalysis += 'exp.createAnalysis()\n'
        strAnalysis += 'exp.setExternalStatus("ANALYZING")\n'
        strAnalysis += 'exp.loadAnalysis("consensus")\n'
        strAnalysis += 'exp.annotateConsensus()\n'
        strAnalysis += 'exp.mapConsensus()\n'
        strAnalysis += 'exp.buildMotifSize()\n\n'
        
        strAnalysis += 'exp.loadAnalysis("1 mismatch")\n'
        strAnalysis += 'exp.annotateConsensus(numMismatches=1)\n'
        strAnalysis += 'exp.mapConsensus(numMismatches=1)\n'
        strAnalysis += 'exp.buildMotifSize()\n\n'
        
        strAnalysis += 'exp.loadAnalysis("PWM - 90% threshold")\n'
        strAnalysis += 'exp.annotateConsensus()\n'
        strAnalysis += 'exp.mapMotifs(90.0)\n'
        strAnalysis += 'exp.buildMotifSize()\n\n'
        
        return strAnalysis + '\n'
    
    def expStatus(self):
        strStatus = ''
        
        strStatus += 'exp.resetExternalStatus()\n'
        
        return strStatus + '\n'

