###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-10 by:                                              #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
__all__ = ["meme", "locator","mafft", "paircomp", "cisGreedy", "gibbs"]

from os import environ

#supportedPrograms = [('alignace','AlignACE'), ('cobind','CoBind'), ('footprinter','FootPrinter'), ('gibbs','Gibbs'), ('locator','Locator'), ('meme','Meme'), ('ymf','Ymf'), ('mafft', 'Mafft'), ('paircomp', 'Paircomp')]
supportedPrograms = [('locator','Locator',{'consensus':'', 'identifier':''}), ('meme','Meme',{'model':'zoops', 'nmotifs':10, 'maxwidth':16}), ('cisGreedy', 'CisGreedy',{'model':'zoops','nmotifs':10,'maxwidth':16,'minwidth':6,'reverse':True,'background':'None', 'iterations':100, 'founder':False,'percentID':75,'Markov size':3}),('gibbs','Gibbs',{'model':'zoops','nmotifs':10,'maxwidth':16,'minwidth':6,'reverse':True,'background':'None','percentID':50,'Markov size':3,'iterations':100})]

class Program:
    """ Program is the Super class for all of the programs supported by cistematic and 
        that are typically called by other classes in cistematic, such as the Experiments.
        Children will overide the methods that they need to instantiate. 
    """
    if environ.get("CISTEMATIC_ROOT"):
    	programRoot = environ.get("CISTEMATIC_ROOT") + '/programs'
    else:
    	programRoot = '/proj/genome/programs'
    contents = ''
    tagID = ''
    
    def __init__(self, tagID='', inputFilePath='', outputFilePath=''):
        """   constructor - inputFilePath is supposed to be data and outputFilePath the results
            although usage can be redefined by children. tagID is a motif string identifier.
        """
        self.tagID = ''
        self.inputFilePath = inputFilePath
        self.outputFilePath = outputFilePath

    def setTagID(self, tid):
        """ set motif string identifier.
        """
        self.tagID = tid
    
    def name(self):
        """ return class name for program.
        """
        return self.__class__.__name__
    
    def inputFile(self, inputFilePath):
        """ set the inputFile
        """
        self.inputFilePath = inputFilePath
    
    def outputFile(self, outputFilePath):
        """ outputFile() - set the outputFile
        """
        self.outputFilePath = outputFilePath

    def setSeqLength(self, length):
        """ only implemented by those programs that need it
        """
        pass
    
    def setGenome(self, genome):
        """ only implemented by programs that support genome defaults
        """
        pass
    
    def setGenExpOptions(self, optionArray):
        """ stub for genExp options.
        """
        pass
    
    def run(self):
        """method is normally implemented by children
        """
        pass
    
    def display(self):
        """ show all of the lines in contents, which holds the program run results.
        """
        for line in self.contents:
            print line
    
    def load(self, inFilePath):
        """ load previously saved data for further processing
        """
        inFile = open(inFilePath, 'r')
        
        self.contents = inFile.readlines()
        
        inFile.close()
    
    def save(self):
        """     save all of contents to outputFilePath
        """
        outFile = open(self.outputFilePath, 'w')
        
        for line in self.contents:
            outFile.write(line)
        
        outFile.close()
