###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-10 by:                                              #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# meme.py
from cistematic.programs import Program
import os, time, string
from cistematic.core.motif import Motif

class Meme(Program):
    memePath = Program.programRoot + '/meme.3.0.8/'
    model = "zoops"
    background = ""
    nmotifs = '10'
    minwidth = '6'
    maxwidth = '15'
    bfile = ''
    motifs = []

    def getSettings(self):
        return (self.model, self.background, self.nmotifs, self.bfile)
    
    def setSettings(self, settings):
        self.motifs = []
        (self.model, self.background, self.nmotifs, self.bfile) = settings
    
    def setGenExpOptions(self, optionArray):
        for option in optionArray:
            (optionTag, optionValue) = option.split(':')
            if optionTag == 'model':
                self.setModel(optionValue)
            if optionTag == 'nmotifs':
                self.setNumMotifs(optionValue)
            if optionTag == 'maxwidth':
                self.setMaxWidth(optionValue)
    
    # command = "meme INO_up800.s -dna -mod tcm -revcomp -bfile yeast.nc.6.freq > ex4.htm"
    def buildCommand(self):
        cmd = self.memePath + 'bin/meme ' + self.inputFilePath + ' -dna -maxsize 1000000' + \
				' -maxw '+ self.maxwidth+\
				' -minw '+ self.minwidth+\
				' -mod ' + self.model + ' -revcomp -nmotifs ' + self.nmotifs + ' '
        
        if self.bfile != '':
            cmd = cmd + ' -bfile ' + self.bfile + ' '
        
        return cmd

    # set motif distribution model
    def setModel(self, modelType):
        if modelType in ['oops', 'zoops', 'tcm']:
            self.model = modelType
    
    # set number of motifs
    def setNumMotifs(self, motifNum):
        self.nmotifs = str(motifNum)
        
    def setMinWidth(self, width):
        self.minwidth = str(width)

    def setMaxWidth(self, width):
        self.maxwidth = str(width)
    
    # set background frequency file
    def setBackground(self, backFile):
        self.bfile = backFile

    # run the program - preferrably after some of the other options have been set
    def run(self):
        startTime = time.time()
        self.contents = os.popen(Meme.buildCommand(self)).readlines()
        stopTime = time.time()
            
        print "\nThis run took %.3f - %.3f = %.3f seconds and produced %d lines" % (startTime, stopTime, stopTime - startTime, len(self.contents))
 
    def getMotifs(self):
        index = 0
        
        self.motifs = []
        try:
            for motif in range(0, int(self.nmotifs)):
                thePWM = []
                theSeqs = []
                consensus = ""
                index = self.locateMotifSeqs(index)
                #index = self.locateMotif(index)
                info = self.contents[index]
                index += 1
                while string.find(self.contents[index],'//') < 0:
                    fields = self.contents[index].split()
                    theSeqs.append(fields[-2])
                    index += 1
                    #values = self.contents[index].split()
                    #thePWM.append([float(values[0]), float(values[1]), float(values[2]), float(values[3])])
                    #index += 1
                self.motifs.append(Motif(self.tagID+ '-meme-' + str(motif + 1),'', thePWM, theSeqs, 0.0, info))
                #self.motifs.append(Motif(self.tagID+ '-meme-' + str(motif + 1),'', thePWM, [], 0.0, info))
        except:
            pass
        return self.motifs
    
    def locateMotif(self, startFrom):
        index = startFrom
        numLines = len(self.contents)
        
        while index < numLines and string.find(self.contents[index],'pspm_doc') < 0:
            index += 1
        
        if index < numLines:
            return index + 3
            
        return -1
        
    def locateMotifSeqs(self, startFrom):
        index = startFrom
        numLines = len(self.contents)
        
        while index < numLines and string.find(self.contents[index],'BLOCKS_doc') < 0:
            index += 1
        
        if index < numLines:
            return index + 3
            
        return -1
        
        

    
        