###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-10 by:                                              #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# parent.py - defines Parent class used by all children classes in cistematic.programs
#
class Parent:
    contents = ''
    tagID = ''
    
    # constructor - inputFilePath is supposed to be data and outputFilePath the results
    #               although usage can be redefined by children
    def __init__(self, tagID='', inputFilePath='', outputFilePath=''):
        self.tagID = ''
        self.inputFilePath = inputFilePath
        self.outputFilePath = outputFilePath

    def setTagID(self, tid):
        self.tagID = tid
    
    def name(self):
        return self.__class__.__name__
    
    # inputFile() - set the inputFile
    def inputFile(self, inputFilePath):
        self.inputFilePath = inputFilePath
    
    # outputFile() - set the outputFile
    def outputFile(self, outputFilePath):
        self.outputFilePath = outputFilePath

    # setSeqLength() - only implemented by those programs who need it
    def setSeqLength(self, length):
        pass
    
    # run() - method will be implemented by children
    def run(self):
        pass
    
    # display() - show all of the lines in contents
    def display(self):
        for line in self.contents:
            print line
    
    # load() - load previously saved data for further processing
    def load(self, inFilePath):
        inFile = open(inFilePath, 'r')
        
        self.contents = inFile.readlines()
        
        inFile.close()
    
    # save() - save all of contents to outputFilePath
    def save(self):
        outFile = open(self.outputFilePath, 'w')
        
        for line in self.contents:
            outFile.write(line)
        
        outFile.close()