###########################################################################
#                                                                         #
# C O P Y R I G H T   N O T I C E                                         #
#  Copyright (c) 2003-10 by:                                              #
#    * California Institute of Technology                                 #
#                                                                         #
#    All Rights Reserved.                                                 #
#                                                                         #
# Permission is hereby granted, free of charge, to any person             #
# obtaining a copy of this software and associated documentation files    #
# (the "Software"), to deal in the Software without restriction,          #
# including without limitation the rights to use, copy, modify, merge,    #
# publish, distribute, sublicense, and/or sell copies of the Software,    #
# and to permit persons to whom the Software is furnished to do so,       #
# subject to the following conditions:                                    #
#                                                                         #
# The above copyright notice and this permission notice shall be          #
# included in all copies or substantial portions of the Software.         #
#                                                                         #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,         #
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF      #
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                   #
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS     #
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN      #
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN       #
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE        #
# SOFTWARE.                                                               #
###########################################################################
#
# data for Saccharomyces cerevisiae
import string
from cistematic.genomes import Genome
from os import environ

if environ.get("CISTEMATIC_ROOT"):
    cisRoot = environ.get("CISTEMATIC_ROOT") 
else:
    cisRoot = '/proj/genome'

geneDB = cisRoot + '/S_cerevisiae/scerevisiae.genedb' 

def loadChromosome(db, chromID, chromPath, chromOut):
    seq = ''
    seqArray = []
    seqLen = 0
    scGenome = Genome('scerevisiae', dbFile=db)
    
    inFile = open(chromPath, 'r')
    # skip the header
    index = 0
    line = inFile.readline()
    for line in inFile:
        seqArray.append(line.strip())
    
    seq = string.join(seqArray,'')
    seqLen = len(seq)
    if seqLen < 1:
        print "Problems reading sequence from file"
    print "writing to file %s" % chromOut
    outFile = open(cisRoot + chromOut,'w')
    outFile.write(seq)
    outFile.close()
    seq = ''
    print "calling scGenome()"
    scGenome.addChromosomeEntry(chromID, chromOut, 'file')
    # Add alternative chromID - should be A-O and 01-09
    #scGenome.addChromosomeEntry('chromo' + chromID, chromOut, 'file')

def loadGeneEntries(db, gFile):
    geneEntries = []
    geneFeatures = []
    scGenome = Genome('scerevisiae', dbFile=db)
    geneFile = open(gFile, 'r')
    
    for line in geneFile:
        field = line.split('\t')
        if field[1] != 'ORF':
            continue
        
        orfName = field[3].strip()
        
        sense = field[11]
        chrom = field[8].strip()
        if sense == 'W':
            sense = 'F'
            try:
                start = int(field[9].strip()) - 1
                stop  = int(field[10].strip()) - 1
            except:
                start = 0
                stop = 0
        else:
            sense = 'R'
            try:
                start = int(field[10].strip()) - 1
                stop  = int(field[9].strip()) - 1
            except:
                start = 0
                stop = 0
        
        length = abs(stop - start)
                
        geneID = ('scerevisiae', orfName)
        gidVersion = 1
        geneEntries.append((geneID, chrom, start, stop, sense, 'chromosomal_feature', gidVersion))
        geneFeatures.append((geneID, gidVersion, chrom, start, stop, sense, 'CDS'))
    
    print "loading %d gene entries" % len(geneEntries)
    scGenome.addGeneEntryBatch(geneEntries)
    print "loading %d gene features" % len(geneFeatures)
    scGenome.addFeatureEntryBatch(geneFeatures)

def loadGeneAnnotations(db, annotPath):
    geneAnnotations = []
    annotFile = open(annotPath, 'r')  
    lines = annotFile.readlines()
    annotFile.close()
    
    scGenome = Genome('scerevisiae', dbFile=db)
    
    for line in lines:
        field = line.split('\t')
        if field[1] != 'ORF':
            continue
        try:
            orfName = field[6].strip()
            description =  field[15].strip()
            geneAnnotations.append((('scerevisiae', orfName), string.replace(description, "'", "p")))
        except:
            pass
    
    print "Adding %d annotations" % len(geneAnnotations)
    scGenome.addAnnotationBatch(geneAnnotations)

def loadGeneOntology(db, goPath, goDefPath):
    scGenome = Genome('scerevisiae', version='SGD1', dbFile=db)
    goDefFile = open(goDefPath, 'r')
    goFile = open(goPath, 'r')
    
    goDefEntries = goDefFile.readlines()
    goDefs = {}
    
    for goDefEntry in goDefEntries:
        if goDefEntry[0] != '!':
            cols = goDefEntry.split('\t')
            goDefs[cols[0]] = (cols[1], cols[2].strip())
    
    goEntries = goFile.readlines()
    
    goArray = []
    for line in goEntries:
        if line[0] == '!':
            continue
        fields = line.split('\t')
        #print fields
        genes = fields[10].split('|')
        gID = genes[0]
        GOID = fields[4]
        objType = fields[11]
        objNameArray = fields[10].split('|')
        objName = objNameArray[0]
        isNot = fields[3]
        try:
            GOterm  = string.replace(goDefs[GOID][0], "'", "p")
        except:
            print 'Could not translate %s' % (GOID)
            GOterm = ''
        evidence = fields[6]
        goArray.append((('scerevisiae', gID), GOID[3:], objType, objName, isNot, GOterm, evidence, fields[1]))
        
    scGenome.addGoInfoBatch(goArray)

def createDBFile(db):
    scGenome = Genome('scerevisiae', version='SGD1',  dbFile=db)
    scGenome.createGeneDB(db)

def createDBindices(db):
    scGenome = Genome('scerevisiae', version='SGD1', dbFile=db)
    scGenome.createIndices()

def buildScerevisiaeDB(db=geneDB):
    genePath = cisRoot + '/download/SGD_features.tab'
    goDefPath = cisRoot + '/download/GO.terms_and_ids'
    goPath = cisRoot + '/download/gene_association.sgd'
    
    chromos = {'1': cisRoot + '/download/chr01.fsa', '2': cisRoot + '/download/chr02.fsa', 
    '3': cisRoot + '/download/chr03.fsa', '4': cisRoot + '/download/chr04.fsa', 
    '5': cisRoot + '/download/chr05.fsa', '6': cisRoot + '/download/chr06.fsa', 
    '7': cisRoot + '/download/chr07.fsa', '8': cisRoot + '/download/chr08.fsa', 
    '9': cisRoot + '/download/chr09.fsa', '10': cisRoot + '/download/chr10.fsa', 
    '11': cisRoot + '/download/chr11.fsa', '12': cisRoot + '/download/chr12.fsa', 
    '13': cisRoot + '/download/chr13.fsa', '14': cisRoot + '/download/chr14.fsa', 
    '15': cisRoot + '/download/chr15.fsa', '16': cisRoot + '/download/chr16.fsa' }
    
    print "Creating database %s" % db
    createDBFile(db)
    
    print "Adding gene entries"
    loadGeneEntries(db, genePath)
    
    print "Adding gene annotations"
    loadGeneAnnotations(db, genePath)
    
    print "Adding gene ontology"
    loadGeneOntology(db, goPath, goDefPath)
    
    for chromID in ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16']:
        print "Loading chromosome %s" % chromID
        loadChromosome(db, chromID, chromos[chromID], '/S_cerevisiae/chr' + chromID + '.bin')
    
    print "Creating Indices"
    createDBindices(db)
    
    print "Finished creating database %s" % db
